function [bhat Hx Fx Sx KMdist knots]=cnskn(x,status, knots,kgon)


% This is an interior function used by the HCNS 
% in the case where 'auto' knots are selected.

%It smooths over the kaplan meier cumulative hazard estimator and yields
%the the corresponding distance (sum of squares of the distance of the 
%spline and the KM cumulative hazard at event times)

%This KMdist distance will be used to compare between the
%knot schemes and select the more appropriate in the HCNS function

%Code Author: Leonidas E. Bantis. (25th January 2012).
%        University of the Aegean, Greece.
%e-mail: leobantis@gmail.com 

%minn=min(x);
%maxx=max(x);


[y x]=ecdf(x,'censoring',status,'function','cumulative hazard');
x1=x;y1=y;t=knots;
%X=[(x-t(1)).^3.*(x>t(1))  (x-t(2)).^3.*(x>t(2))  (x-t(3)).^3.*(x>t(3)) (x-t(4)).^3.*(x>t(4)) (x-t(5)).^3.*(x>t(5))  ];

     
     X=zeros(length(x),(length(t)-1)); %preallocate
     for i=1:(length(t)-1)
           X(:,i)=(x-t(i)).^3.*(x>t(i));
     end
            

options = optimset('LargeScale', 'off','MaxFunEvals',18000,'MaxIter',18000);
[A,k,Aeq]=conlsqlin(t,kgon); %find the required constraints using the knots "t".
[bhat] = lsqlin(X,y,A,(k),Aeq,0,[],[],[],options);

g=bhat;
% The last parameter can be explicitly calculated by:
g(6)=(g(1).*(t(1)-t(5))+g(2).*(t(2)-t(5))+g(3).*(t(3)-t(5))+g(4).*(t(4)-t(5)))./(t(5)-t(6));
%g(length(bhat)+1)= ([g(1:(end-1))]') * (t(1:(end-2))-t(end-1))' / (t(end-1)-t(end));
 %--------------------------------------------
 %DONE WITH ESTIMATING THE SPLINE COEFFICIENTS
 %--------------------------------------------
  t=t';

  Hx=@(x)  Hazx(x,t,g);
  Fx=@(x)  1-exp(-(Hazx(x,t,g)));
  Sx=@(x)  1-Fx(x);
 
KMdist=sum((Hx(x1)-y1').^2);
bhat=g;
knots=t(1:end);

end


%----Function to evaluate the cumulative hazard 

function out=Hazx(x,t,g)
%Needs the point at which you wish to evaluate the H(x) ("x")
%Needs the knots "t"
%And the estimated coefs of the spline "g"
sizex=size(x);if sizex(1)==1;x=x';end
Hazx = @(x) sum(bsxfun(@times,g(:),bsxfun(@minus,x',t(:)).^3 .*bsxfun(@gt,x',t(:))),1);
out=Hazx(x);
   if isnan(out)==1;out=Inf;end         
end


